@@ -514,7 +514,7 @@ bacon_volume_scale_new (BaconVolumeButton * button,
   BaconVolumeScale *scale = g_object_new (BACON_TYPE_VOLUME_SCALE, NULL);
   GtkObject *adj;
 
-  adj = gtk_adjustment_new (min, min, max, -step, 10 * -step, 0);
+  adj = gtk_adjustment_new (min, min, max, step, 10 * step, 0);
   gtk_range_set_adjustment (GTK_RANGE (scale), GTK_ADJUSTMENT (adj));
   scale->button = button;
 
@@ -586,12 +586,10 @@ bacon_volume_scale_value_changed (GtkRange * range)
   GdkPixbuf *buf;
 
   if (val == adj->lower)
-    s = "stock_volume-mute";
-  else if (val > adj->lower && val <= adj->lower + step)
     s = "stock_volume-0";
-  else if (val > adj->lower + step && val <= adj->lower + step * 2)
+  else if (val > adj->lower && val <= adj->lower + step)
     s = "stock_volume-min";
-  else if (val > adj->lower + step * 2 && val <= adj->lower + step * 3)
+  else if (val > adj->lower + step && val <= adj->lower + step * 2)
     s = "stock_volume-med";
   else
     s = "stock_volume-max";
@@ -600,6 +598,7 @@ bacon_volume_scale_value_changed (GtkRange * range)
   gtk_icon_size_lookup (button->size, &w, &h);
   buf = gtk_icon_theme_load_icon (button->theme, s, w, 0, NULL);
   gtk_image_set_from_pixbuf (GTK_IMAGE (button->image), buf);
+  g_object_unref (buf);
 #endif
 
   /* signal */
